!< \file beta_iter_vitbil_mod.f90
!< Module dragging pour affiner le beta a partir d'une carte vitbil  sur les noeuds centres.
!< A appeler depuis steps_time_loop
!< a utiliser avec beta_iter_vitbil_mod


module dragging_beta_iter_vitbil_mod

  use module3d_phy
  use interface_input
  use netcdf
  use io_netcdf_grisli
  use beta_iter_vitbil_mod

  implicit none


!~   real, dimension(nx,ny) :: Vcol_x           !< uniquement pour compatibilite
!~   real, dimension(nx,ny) :: Vcol_y           !<
!~   real, dimension(nx,ny) :: Vsl_x            !<
!~   real, dimension(nx,ny) :: Vsl_y            !<


!~   real, dimension(nx,ny) :: drag_centre      !< beta on major node (average)
!~   real :: beta_limgz                         !< when beta gt beta_limgz -> not gzmx
!~   real :: beta_min                           !< minimum value of beta
!~   real :: beta_mult                          !< coefficient of beta field
!~   integer  :: ill,jll,nmoy

!~   real :: maxi                               !< calcul correction dS a la grounding line
!~   real :: mini

!~   logical :: corr_def = .False.               !< for deformation correction (compatibility)


!~   real, dimension(nx,ny) :: Umag_vitbil             !< vitesse de bilan
!~   real, dimension(nx,ny) :: Uslid_vitbil            !< vitesse de bilan partie glissement
!~   real, dimension(nx,ny) :: Umag_direct             !< vitesse moyenne centree calculee par GRISLI
!~   real, dimension(nx,ny) :: Uslmag_direct           !< vitesse glissement centree calculee par GRISLI
!~   real, dimension(nx,ny) :: Udefmag_direct          !< vitesse deformation centree calculee par GRISLI
!~   real, dimension(nx,ny) :: driving_stress          !< driving stress

!~   character(len=100)     :: Umag_bil_file           !< fichier qui contient les donnees
!~   real                   :: time_iter               !< temps de debut des iterations
!~   real                   :: time_iter_end           !< temps de fin des iterations
!~   real                   :: time_reiter             !< nbr annees entre 2 iterations calcul beta
!~   integer                :: nb_iter_vitbil          !< nombre d'iteratiosn avant de changer de pas de temps
!~   real                   :: coef_iter_vitbil        !< coefficient <= 1 (rapport vitesses)
!~   real                   :: J_Umag                  !< estimateur ecart entre vitesses
!~   real                   :: J_Udef                  !< estimateur vitesse regions sans glissement
!~   integer                :: nb_umag                 !< nombre de points calcul J_Umag
!~   integer                :: nb_Udef                 !< nombre de points calcul J_Umag


contains

  !-----------------------------------------------------------------------------------------
  !> SUBROUTINE: init_dragging 
  !! Cette routine fait l'initialisation du dragging.
  !<
  subroutine init_dragging      ! Cette routine fait l'initialisation du dragging.
    ! nouvelle version : lit les fichiers nc
    implicit none
    character(len=100) :: beta_c_file               !  beta on centered grid
    character(len=100) :: betamx_file               !  beta on staggered grid mx
    character(len=100) :: betamy_file               !  beta on staggered grid mx
    real*8, dimension(:,:),   pointer  :: tab => null() !< tableau 2d real ecrit dans le fichier pour lect ncdf

    namelist/beta_prescr/beta_c_file,beta_limgz,beta_min,beta_mult

    if (itracebug.eq.1)  call tracebug('beta_iter_vitbil_mod subroutine init_dragging')

    inv_beta=1

    ! lecture des parametres du run                    
    !--------------------------------------------------------------------

    rewind(num_param)        ! pour revenir au debut du fichier param_list.dat
428 format(A)
    read(num_param,beta_prescr)

    write(num_rep_42,428) '!___________________________________________________________' 
    write(num_rep_42,428) '!  read beta on centered grid'
    write(num_rep_42,beta_prescr)                    
    write(num_rep_42,428) '!  beta_file : nom des fichier qui contiennent les beta_c'
    write(num_rep_42,428) '!  above beta_limgz, gzmx is false'
    write(num_rep_42,428) '!  if grounded, beta_min minimum value '
    write(num_rep_42,428) '!  beta_mult : coefficient just after reading' 
    write(num_rep_42,*)
    write(num_rep_42,428) '!___________________________________________________________' 


    beta_c_file = trim(dirnameinp)//trim(beta_c_file)
    betamx_file = trim(dirnameinp)//trim(betamx_file)
    betamy_file = trim(dirnameinp)//trim(betamy_file)
    betamax = beta_limgz
    betamax_2d(:,:) = betamax

!    call lect_input(1,'beta_c',1,drag_centre,beta_c_file,'')
    write(6,*)  beta_c_file

    call Read_Ncdf_var('z',beta_c_file,tab)
    drag_centre(:,:) = tab(:,:)


    ! multiplication par le coefficient beta_mult

    drag_centre(:,:) = drag_centre(:,:)*beta_mult

    where (mk_init(:,:).eq.-2)                                             ! iles a probleme
       drag_centre(:,:) = 2.* abs(beta_limgz)
    end where

    where(flot(:,:))
       drag_centre(:,:) = 0.
    end where


    call  beta_c2stag(nx,ny,drag_mx,drag_my,drag_centre)         ! redistribute on staggered grid

    if  (beta_limgz.gt.0.) then
       beta_centre(:,:) =  drag_centre(:,:)
       betamx(:,:)      =  drag_mx(:,:)
       betamy(:,:)      =  drag_my(:,:)

    else if (beta_limgz.lt.0.) then                              ! attention sans doute pour plastic

       beta_centre(:,:) = - beta_limgz
       betamx(:,:)      = - beta_limgz
       betamy(:,:)      = - beta_limgz
       drag_centre(:,:) = - beta_limgz 
       drag_mx(:,:)     = - beta_limgz
       drag_my(:,:)     = - beta_limgz 
       beta_limgz = 1.
    end if



    call beta_min_value(beta_min)      !  valeur mini que peut avoir beta (en Pa an m-1)
    ! on peut envisager une valeur ~ 10 
    ! rappel : beta doit être positif.



    !    call beta_c2stag(nx,ny,drag_mx,drag_my,drag_centre)                  ! centered distributed
    !                                                                         ! on staggered
    !


    beta_centre(:,:) =  drag_centre(:,:)                                 ! surtout pour sorties



    where (drag_mx(:,:).ge.beta_limgz)
       mstream_mx(:,:) = 0
       betamx(:,:)     = beta_limgz
       drag_mx(:,:)    = beta_limgz
    elsewhere
       mstream_mx(:,:) = 1
       betamx(:,:)     = drag_mx(:,:)
    endwhere

    where (drag_my(:,:).ge.beta_limgz)
       mstream_my(:,:) = 0
       betamy(:,:)     = beta_limgz
       drag_my(:,:)    = beta_limgz
    elsewhere
       mstream_my(:,:) = 1
       betamy(:,:)     = drag_my(:,:)
    endwhere

    if (itracebug.eq.1)  call tracebug(' Prescr_beta  apres lecture')
    mstream_mx(:,:) = 0
    mstream_my(:,:) = 0


    call gzm_beta_prescr

    call init_beta_iter_vitbil
    return

  end subroutine init_dragging

  !________________________________________________________________________________

  !> SUBROUTINE: dragging 
  !! Defini la localisation des streams et le frottement basal
  !<
  !-------------------------------------------------------------------------
  subroutine dragging   ! defini la localisation des streams et le frottement basal

    implicit none

    if (abs(time-time_iter_end).lt.dtmin) then
      time_iter = time_iter + time_reiter
      time_iter_end = time_iter_end + time_reiter
      print*,'debug dragging beta_iter_vitbil_mod time_iter=', time_iter, time_iter_end, time
    endif

    if (itracebug.eq.1)  call tracebug(' beta_prescr         subroutine dragging')


    betamx(:,:)=drag_mx(:,:)
    betamy(:,:)=drag_my(:,:)


    call  gzm_beta_prescr        ! determine gz et flgz et met a 0 le beta des points flottants


    return
  end subroutine dragging

  !----------------------------------------------------------------------------------
  !>SUBROUTINE: gzm_beta_prescr
  !! Calcul de gzmx
  !<

  subroutine gzm_beta_prescr     ! attribue flgzmx et gzmx (et my)
    
    logical ::  test_Tx          ! test if there is a basal melting point in the surrounding
    logical ::  test_Ty          ! test if there is a basal melting point in the surrounding
    logical ::  test_beta_x      ! test if there is a low drag point in the surrounding
    logical ::  test_beta_y      ! test if there is a low drag point in the surrounding

    real    ::  beta_forc_fleuve ! below this value -> ice stream

!    beta_forc_fleuve = 5.e3
    beta_forc_fleuve = beta_limgz

    ! calcul de gzmx 


    gzmx(:,:)=.true.
    gzmy(:,:)=.true.
    flgzmx(:,:)=.false.
    flgzmy(:,:)=.false.


    ! points flottants : flgzmx mais pas gzmx
    do j=2,ny
       do i=2,nx
          if (flot(i,j).and.(flot(i-1,j))) then
             flotmx(i,j)=.true.
             flgzmx(i,j)=.true.
             gzmx(i,j)=.false.
             betamx(i,j)=0.
          end if
          if (flot(i,j).and.(flot(i,j-1))) then
             flotmy(i,j)=.true.
             flgzmy(i,j)=.true.
             gzmy(i,j)=.false.
             betamy(i,j)=0.
          end if
       end do
    end do

    if (itracebug.eq.1)  call tracebug(' apres criteres flot')

    if (itracebug.eq.1) write(num_tracebug,*) 'gzmx', gzmx(127,330) 

    !--------- autres criteres

    ! points poses
    gzmx(:,:)=gzmx(:,:).and.(betamx(:,:).lt.beta_limgz)   !  Pas de calcul pour les points
    gzmy(:,:)=gzmy(:,:).and.(betamy(:,:).lt.beta_limgz)   !  au fort beta



    ! critere (lache) sur la temperature
    do j = 2, ny-1
       do i =2, nx-1 

!   test s'il y a un point tempere dans les environs
          test_Tx = any( ibase( i-1:i   , j-1:j+1 ).gt.1)
          test_Ty = any( ibase( i-1:i+1 , j-1:j   ).gt.1)

!   test s'il y a un point low drag dans les environs
          test_beta_x = any( drag_centre( i-1:i   , j-1:j+1 ) .lt. beta_forc_fleuve )
          test_beta_y = any( drag_centre( i-1:i+1 , j-1:j   ) .lt. beta_forc_fleuve )

!   critere pour gzmx

! Attention : les deux lignes suivants sont en commentaire pour voir l'effet

!          gzmx(i,j) =  gzmx(i,j) .and. (test_Tx.or.test_beta_x)
!          gzmy(i,j) =  gzmy(i,j) .and. (test_Ty.or.test_beta_y)

       end do
    end do


    if (itracebug.eq.1)  call tracebug(' apres autres criteres ')
    if (itracebug.eq.1) write(num_tracebug,*) 'gzmx', gzmx(127,330) 

    flgzmx(:,:) = flgzmx(:,:) .or. gzmx(:,:)
    flgzmy(:,:) = flgzmy(:,:) .or. gzmy(:,:)

    where ((.not.flotmx(:,:)).and.(.not.gzmx(:,:)))
       betamx(:,:) = beta_limgz
    end where

    where ((.not.flotmy(:,:)).and.(.not.gzmy(:,:)))
       betamy(:,:) = beta_limgz
    end where

	fleuvemx(:,:)=gzmx(:,:)
	fleuvemy(:,:)=gzmy(:,:)
   
  end subroutine gzm_beta_prescr


!-----------------------------------------------------------------------------------------
end module dragging_beta_iter_vitbil_mod
